from .consulta import (
    CDA,
    OAB,
    CheckResults,
    ConsultaPor,
    ConsultaProcesso,
    Consultar,
    DocumentoParte,
    NomeAdvogado,
    NomeParte,
    NumCartaPrecatoria,
    NumDocDelegacia,
    NumeroUnificado,
)
from .registro import Header, Partes, Movimentacao

# import .registro
