"""

"""

from .scraper import params
# from .scraper import page, webdriver, search, outros

#
#
# from .page.cadastro import DadosCadastrais, GerenciamentoPerfis
# from .page.consulta_processos import (
#     CDA,
#     OAB,
#     ConsultaPor,
#     ConsultaProcesso,
#     Consultar,
#     DocumentoParte,
#     NomeAdvogado,
#     NomeParte,
#     NumCartaPrecatoria,
#     NumDocDelegacia,
#     NumeroUnificado,
# )
# from .page.login import Login
# from .page.recebe_initimacoes import (
#     Acoes,
#     Area,
#     AssuntoPrincipal,
#     Cargo,
#     CheckError,
#     Classe,
#     ConsultarIntimacoes,
#     EmNomeDe,
#     Especializacao,
#     Foro,
#     Instancia,
#     Intimacoes,
#     NaturezaComunicacao,
#     OrgaoJulgador,
#     Paginacao,
#     Processo,
#     Secao,
#     Tabela,
#     TipoParticipacao,
#     Vara,
# )
